/*
 * National Center for SuperComputing Applications, University of Illinois
 *
 * This NCSA software product is public domain software.  Permission
 * is hereby granted to do whatever you like with it. Should you wish
 * to make a contribution towards the production of this software, please
 * send us your comments about your experience with the software,  why
 * you liked or disliked it, how you use it, and most importantly, how it
 * helps your work. We will receive your comments at softdev@ncsa.uiuc.edu.
 *
 * Please send bug reports to bugs@ncsa.uiuc.edu
 *
 * Author: Eng-Whatt Toh, National Center for Supercomputing Applications
 *         ewtoh@ncsa.uiuc.edu
 */
/* $Id: gr_xwi.h,v 1.2 1993/07/15 19:33:54 gbourhis Exp $ */

/*
 *	File:		gr_xwi.h
 *	Contents:	Header file containing all the includes and externs
 *				by the X-Windows Athena and HP widget sets.
 */

/* add prefix X11/ to the X11 include. gbourhis Jan 93	*/
#include <X11/Xos.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>

#ifdef XtSpecificationRelease
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/List.h>
#include <X11/Xaw/Viewport.h>
#else
#include <X11/AsciiText.h>
#include <X11/Box.h>
#include <X11/Command.h>
#include <X11/Dialog.h>
#include <X11/Label.h>
#include <X11/List.h>
#include <X11/Viewport.h>
#endif

/* Avoid compiler redefinition warnings */
#ifdef XtSpecificationRelease
#undef XtNhighlightThickness
#undef XtNcursor
#undef XtRTranslationTable
#undef XtCColumns
#undef XtNdisplayPosition
#undef XtNleftMargin
#undef XtNrightMargin
#undef XtNtopMargin
#undef XtNbottomMargin
#undef XtNtextSource
#undef XtNselection
#undef XtNeditType
#undef XtNfile
#undef XtNstring
#undef XtNlength
#undef XtNfont
#undef XtNwrap
#undef XtCWrap
#undef XtCScroll
#endif

#include <Xw/Xw.h>
#include <Xw/Arrow.h>
#include <Xw/BBoard.h>
#include <Xw/RCManager.h>
#include <Xw/ScrollBar.h>
#include <Xw/SRaster.h>
#include <Xw/TitleBar.h>
#include <Xw/Toggle.h>
#include <Xw/SWindow.h>
#include <Xw/Valuator.h>
#include <Xw/WorkSpace.h>

extern Arg	argList[15];
