/*
 * National Center for SuperComputing Applications, University of Illinois
 *
 * This NCSA software product is public domain software.  Permission
 * is hereby granted to do whatever you like with it. Should you wish
 * to make a contribution towards the production of this software, please
 * send us your comments about your experience with the software,  why
 * you liked or disliked it, how you use it, and most importantly, how it
 * helps your work. We will receive your comments at softdev@ncsa.uiuc.edu.
 *
 * Please send bug reports to bugs@ncsa.uiuc.edu
 *
 * Author: Eng-Whatt Toh, National Center for Supercomputing Applications
 *         ewtoh@ncsa.uiuc.edu
 */
/* $Id: td_var.h,v 1.2 1993/07/15 19:50:27 gbourhis Exp $ */

/*
 *	File:		td_var.h
 *	Contents:	Header file containing all global variables used
 *				by the td module.
 */

static	char	*td_toolName="NCSA X DataSlice";
static	char	*td_relType=""; /* "normal release" gbourhis Feb 93 */
static	char	*td_dirDefName=".";

extern	char	*td_logFile;  /* will be init to $HOME/xdsLog */
extern	char	*td_dirName;
extern	double	td_version;
extern	int	td_release;
extern	time_t	lastDirMod;
