/*
 * National Center for SuperComputing Applications, University of Illinois
 *
 * This NCSA software product is public domain software.  Permission
 * is hereby granted to do whatever you like with it. Should you wish
 * to make a contribution towards the production of this software, please
 * send us your comments about your experience with the software,  why
 * you liked or disliked it, how you use it, and most importantly, how it
 * helps your work. We will receive your comments at softdev@ncsa.uiuc.edu.
 *
 * Please send bug reports to bugs@ncsa.uiuc.edu
 *
 * Author: Eng-Whatt Toh, National Center for Supercomputing Applications
 *         ewtoh@ncsa.uiuc.edu
 */
/* $Id: gr_var.h,v 1.2 1993/07/15 19:31:33 gbourhis Exp $ */

/*
 *	File:		gr_var.h
 *	Contents:	Header file containing all global variables used
 *				by the graphics module.
 */

extern Widget		gr_topLevel;			/* Top Level Widget */
extern A_TopWind_t	gr_topWin;			/* Top Window */
extern char		*gr_palData;			/* Palette data */
extern char		gr_DirName[STRNG160];		/* Directory name */
extern char		gr_FileName[STRNG160];		/* File name */
extern A_Color_t	gr_color;			/* Color structure for palette */
extern A_Color_t	gr_colorSplit;			/* Color structure for split pal */
extern ApplicationData	gr_Data;			/* Resources for X DataSlice */
extern XEvent		report;
